/*
	SHT11 test project for PIC18F2550
	clock: 20 MHz crystal
	You may have to change delays in SHTxx.c when changing core clock.
	RB3 - LED (TX_EN_On/Off)
	SHT11 connected to PB5 (DATA, with 10k pull-up) and PB4 (CLOCK),
	connection configured in SHTxx.c.
	With fusebits used I have no problem debugging with PICKIT 2 clone.
*/

#pragma config FOSC = HSPLL_HS
#pragma config PLLDIV   = 5       // (20 MHz input)
#pragma config CPUDIV = OSC4_PLL6
#pragma config USBDIV = 2
#pragma config WDT = OFF
#pragma config BOR = ON
#pragma config BORV = 3			
#pragma config LVP = OFF

#include <p18cxxx.h>
#include <delays.h>
#include <stdlib.h>
#include <string.h>
#include "io_cfg.h"
#include "sht11/include/SHTxx.h"

void main(void)
{
	value humid_val = 0, temp_val = 0;
	unsigned char checksum, error = 0;
	char buf[50] = {0}, buf2[20] = {0};	
	const rom char *strTemp = "Temp: ";
	const rom char *strMinus = "-";
	const rom char *strDot = ".";
	const rom char *strRh = "C, RH: ";
	const rom char *strPercent = "%";


	mInitPB();
	while(1)
	{
		TX_EN_On();
		Delay10KTCYx(200);	
		TX_EN_Off();
		Delay10KTCYx(200);	

		s_connectionreset();
		
		//WARNING! s_measure may take up to 2s (max timeout), so you'll probably
		//have to rewrite this function into non-blocking code if your application does sth time-critical in main loop
		//(it's probably not compatible with cooperative multitasking)
		error+=s_measure((unsigned char*) &temp_val.i,&checksum,TEMP);
		error+=s_measure((unsigned char*) &humid_val.i,&checksum,HUMID);
		if(error != 0)
		{ 
		   s_connectionreset();            
		   //usart_puts("MEAS ERROR\r\n");
		}
		else
		{
		   	humid_val.f=(float)humid_val.i;
		   	temp_val.f=(float)temp_val.i;
		   	calc_sht11(&humid_val.f,&temp_val.f);
		   	temp_val.i=(long)(temp_val.f*10);
		   
		   	strcpypgm2ram(buf, (const rom far char *)strTemp);
		   	if(temp_val.i < 0)
			{
				strcatpgm2ram(buf, (const rom far char *)strMinus);
				temp_val.i = -temp_val.i;
			}
		   	itoa((int)(temp_val.i/10), buf2);
			strcat(buf, buf2);
			strcatpgm2ram(buf, (const rom far char *)strDot);	
			itoa((int)(temp_val.i%10), buf2);
			strcat(buf, buf2);
			strcatpgm2ram(buf, (const rom far char *)strRh);
		   
		   	humid_val.i = (long)(humid_val.f);
		   	itoa((int)(humid_val.i), buf2);
		   	strcat(buf, buf2);
			strcatpgm2ram(buf, (const rom far char *)strPercent);  
			
			//buf contains string with temperature/humidity
			//there is no output - I just tested it with PICKIT 2 debugger
		}	
	}
}
